#!/usr/bin/perl
#
# search for song tracks
#
# imports:
#    environment:
#    TUNEBOT_ROOT    root of tunebot installation.
#    TUNEIX          index of tune files.
#    TUNETAGS        index of tune attributes.
#    BANNER_STRING   identifying string for web page titles
#    QUERY_STRING    string from the browser
#
#    files:
#       $TUNEBOT_ROOT/tune.autotags
#       $TUNEBOT_ROOT/html/bodypart.html.head
#       $TUNEBOT_ROOT/html/bodypart.html.tail
#

$tunebot_root=$ENV{TUNEBOT_ROOT};
$debug=0;

sub urldec {
    my ($a) = $_[0];
    my ($b) = "";
    $a =~ tr/+/ /;
    while ($a ne "") {
        my ($c) = substr($a, 0, 1);
        if ($c eq "%") {
            $b .= chr hex substr($a, 1, 2);
            $a =~ s/...//;
        } else {
            $b .= $c;
            $a =~ s/.//;
        }
    }
    return $b;
}

sub urlenc {
    my ($a)=$_[0];
    $a =~ s/&/&amp;/g;
    $a =~ s/</&lt;/g;
    $a =~ s/>/&gt;/g;
    return $a;
}

#
# take track name and render into nice format in html.
#
sub friendlytitle {
    my ($title) = $tags{@_, "N"};
    if ($title eq "") {
	$title = "<em>" . $title . "</em>";
    } else {
	$title = "<a alt=PlayIt href=\"/cgi-bin/tuneput.cgi?" . @_[0] . ".mp3\">" . $title . "</a>";
	my ($art) = $tags{@_, "ARTIST"};
	if ($art ne "") {
	    $title = $title . " <em>- " . $art . "</em>";
	}
	my ($b) = @_;
	my ($i) = rindex($b, "/");
	if ($i >= 0) {
	    $b = substr($b, 0, $i);
	}
	my ($a) = $b;
	$i = rindex($a, "/");
	if ($i >= 0) {
	    $a = substr($a, 0, $i);
	}
	$i = $tags{$a, "N"};
	if ($i ne "") {
#	    $title = $title . "<br><em>&nbsp;" . $i;
	    my $x = $autotags{$a, "H_ARTIST"};
	    my $y = $autotags{$a, "A_ARTIST"};
	    if ($x eq "" || $y eq "") {
		$title = $title . "<br><em>&nbsp;" . $i;
	    } else {
		$title = $title . "<br><em>&nbsp;<a href=\"/rfm/tunesbyartist" . $x . ".html#" . $y . "\">" . $i . "</a>";
	    }
	    $i = $tags{$b, "N"};
	    if ($i ne "") {
#		$title = $title . " / " . $i;
		my $z = $autotags{$b, "A_ALBUM"};
		if ($x eq "" || $z eq "") {
		    $title = $title . " / " . $i;
		} else {
		    $title = $title . " / <a href=\"/rfm/tunesbyartist" . $x . ".html#" . $z . "\">" . $i . "</a>";
		}
	    }
	    $title = $title . "</em>";
	}
    }
    return $title;
}

sub gettime {
    my ($i) = $tags{@_, "T"};
    if ($i eq "") {
	$i = "?";
    }
    $i = "(" . $i . ")";
    return $i;
}

#
# read all track tags and auto-generated tags into a hash
#
sub suckintags {
    open(TUNETAGS, $ENV{TUNETAGS});
    while ($line = <TUNETAGS>) {
	chop $line;
	$i = index($line, ":");
	$j = index($line, "=", $i);
	$file = substr($line, 0, $i);
	$tag = substr($line, $i + 1, $j - $i - 1);
	$value = substr($line, $j + 1);
	$tags{$file, $tag} = $value;
    }
    close(TUNETAGS);
    open(AUTOTAGS, $ENV{TUNEBOT_ROOT} . "/tune.autotags");
    while ($line = <AUTOTAGS>) {
	chop $line;
	$i = index($line, ":");
	$j = index($line, "=", $i);
	$file = substr($line, 0, $i);
	$tag = substr($line, $i + 1, $j - $i - 1);
	$value = substr($line, $j + 1);
	$autotags{$file, $tag} = $value;
    }
    close(AUTOTAGS);
}

print "Content-type: text/html\n";
print "\n";
print "<html>\n";
print "<META HTTP-EQUIV=\"Pragma\" CONTENT=\"no-cache\">\n";
print "<head>\n";
print "<title>" . $ENV{BANNER_STRING} . " Search</title>\n";
print "</head>\n";
open(I, $tunebot_root . "/html/bodypart.html.head");
while ($line = <I>) {
    print $line;
}
close(I);

if ($debug > 0) {
    print "<p>\n";
    print "query string is: " . $ENV{QUERY_STRING} . "\n";
    print "<p>\n";
    print "i've got " . $#ARGV . " command args\n";
    for ($i = 0; $i <= $#ARGV; $i++) {
	print "<br>" . $i . ". " . @ARGV[$i] . "\n";
    }
    print "<p>\n";
    print "<pre>\n";
}
#
# parse stdin into a list of name-value pairs and stuff into hash %posts.
# this is used to support the POST method.
# stdin format is "name1=value1&name2=value2&...".
#
while ($line = <>) {
    chop $line;
    @nvpairs = split('&', $line);
    for ($i = 0; $i <= $#nvpairs; $i++) {
	@nv = split('=',@nvpairs[$i]);
	if ($#nv == 1) {
	    if ($debug > 0) {
		print @nv[0] . "  =  " . @nv[1] . "\n";
	    }
	    $posts{@nv[0]} = urldec(@nv[1]);
	}
    }
};
#
# parse QUERY_STRING into nv pairs and hash them.
# this supports the GET method (which is currently used).
#
@nvpairs = split('&', $ENV{QUERY_STRING});
for ($i = 0; $i <= $#nvpairs; $i++) {
    @nv = split('=',@nvpairs[$i]);
    if ($#nv == 1) {
	if ($debug > 0) {
	    print @nv[0] . "  =  " . @nv[1] . "\n";
	}
	$posts{@nv[0]} = urldec(@nv[1]);
    }
}
if ($debug > 0) {
    print "</pre>\n";
}

#
# render search command form.
#
print "<p>\n";
print "<center><font size=+2><b>" . $ENV{BANNER_STRING} . " Searcheroo</b></font></center>\n";
print "<p>\n";
print "<form action=\"/cgi-bin/search.cgi\">\n";
print "<b>Find Song Titles with: </b> ";
if ($posts{"titlepat"} ne "") {
    print "<input align=middle name=titlepat type=text value=\"" . urlenc($posts{titlepat}) . "\">\n";
} else {
    print "<input align=middle name=titlepat type=text>\n";
}
#print " <input name=dotitle type=image src=\"/rfm/readthis.gif\" alt=\"Do Search\" align=center border=0>\n";
print "</form>\n";
print "<p>\n";
print "<form action=\"/cgi-bin/search.cgi\">\n";
print "<b>Find with Artist Name: </b> ";
if ($posts{"artistpat"} ne "") {
    print "<input align=middle name=artistpat type=text value=\"" . urlenc($posts{artistpat}) . "\">\n";
} else {
    print "<input align=middle name=artistpat type=text>\n";
}
print "</form>\n";

#
# for sorting upper and lowercase together
#
sub folded { uc($a) cmp uc($b); }

#
# do actual search, if we were called with form data
# this is the by-track-title search.
#
print "<p>\n";
print "<hr>\n";
if ($posts{"titlepat"} ne "") {
    suckintags();
    $re = $posts{titlepat};
    $re =~ tr/A-Z/a-z/;
    open(I, $ENV{TUNEIX});
    $x = 1;
    while ($path = <I>) {
	chop $path;
	$path =~ s/\.mp3$//;
	$n = $tags{$path, "N"};
	$n =~ tr/A-Z/a-z/;
	if ($n =~ $re) {
	    $sortk{$tags{$path, "N"} . "\t$x"} = $path;
	    $x++;
	}
    }
    close(I);

    if ($x == 1) {
	print "<center>No matches were found</center>\n";
    } else {
	print "<center>Click to play song.  Artist and album names take you to the index.</center>\n";
	print "<p><table width=90%>\n";
	$x = 1;
	foreach $sk (sort folded keys %sortk) {
	    $path = $sortk{$sk};
	    print "<tr><td width=10%>" . $x . ".</td>";
	    print "<td width=60%>" . friendlytitle($path) . "</td>";
	    $time = gettime($path);
	    print "<td><a href=\"/cgi-bin/abouttrack.cgi?$path.mp3\">" . gettime($path). "</a> $host</td>\n";
	    print "</tr>\n";
	    $x++;
	}
	print "</table>\n";
    }
}

if ($posts{"artistpat"} ne "") {
    suckintags();
    $re = $posts{artistpat};
    $re =~ tr/A-Z/a-z/;
    open(I, $ENV{TUNEIX});
    $x = 1;
    while ($path = <I>) {
	chop $path;
	$path =~ s/\.mp3$//;
	$a = $tags{$path, "ARTIST"};
	$a =~ tr/A-Z/a-z/;
#
# test ARTIST tag of track,
# then artist directory level Name,
# then album level ARTIST tag.
#
	if ($a =~ $re) {
	    $sortk{$tags{$path, "N"} . "\t$x"} = $path;
	    $x++;
	} else {
	    $apath = $path;
	    $apath =~ s/\/[^\/]*\/[^\/]*$//;
	    $a = $tags{$apath, "N"};
	    $a =~ tr/A-Z/a-z/;
	    if ($a =~ $re) {
		$sortk{$tags{$path, "N"} . "\t$x"} = $path;
		$x++;
	    } else {
		$apath = $path;
		$apath =~ s/\/[^\/]*$//;
		$a = $tags{$apath, "ARTIST"};
		$a =~ tr/A-Z/a-z/;
		if ($a =~ $re) {
		    $sortk{$tags{$path, "N"} . "\t$x"} = $path;
		    $x++;
		}
	    }
	}
    }
    close(I);

    if ($x == 1) {
	print "<center>No matches were found</center>\n";
    } else {
	print "<center>Click to play song.  Artist and album names take you to the index.</center>\n";
	print "<p><table width=90%>\n";
	$x = 1;
	foreach $sk (sort folded keys %sortk) {
	    $path = $sortk{$sk};
	    print "<tr><td width=10%>" . $x . ".</td>";
	    print "<td width=60%>" . friendlytitle($path) . "</td>";
	    $time = gettime($path);
	    print "<td><a href=\"/cgi-bin/abouttrack.cgi?$path.mp3\">" . gettime($path). "</a> $host</td>\n";
	    print "</tr>\n";
	    $x++;
	}
	print "</table>\n";
    }
}

open(I, $tunebot_root . "/html/bodypart.html.tail");
while ($line = <I>) {
    print $line;
}
close(I);

print "</html>\n";
exit 0

